;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xd309b690419ed4f1727ef43626e0f799 . "./code.cg2js/cg-acache.fasl")
(#xd79fa775afa0d5bf0c3790c4e765baae . "./code.cg2js/cg-base.fasl")
(#x1b9fc31a264e51231b8af6816db0a327 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x883cb8606fb293810c8ec3cbbbdc8f88 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x5356242f7a81c775deca8efadc05971f . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc277db5c22dbb864661326ec2a48ef6c . "./code.cg2js/cg-button.fasl")
(#x666141a363dbd3a6152fef5c692a5121 . "./code.cg2js/cg-calendar.fasl")
(#x4e6ca74aa26cffe29245f2085b389585 . "./code.cg2js/cg-caret.fasl")
(#x6d5bc43cd7a9422302aa1859f6428640 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xc7b03a44edc68b49877170e9ab1f2b6c . "./code.cg2js/cg-chart-widget.fasl")
(#xcf86a4b9fca869f29970a6c6debbf883 . "./code.cg2js/cg-check-box.fasl")
(#xb25ef31ad3d13474c9bbd9f8e32e3490 . "./code.cg2js/cg-choice-list.fasl")
(#xa7850fdd2542f872f8ec6c5c70b1ce40 . "./code.cg2js/cg-choose-printer.fasl")
(#xb65094938d775e214ce50575552ac1ee . "./code.cg2js/cg-class-grid.fasl")
(#x47b1e31699ba6b96b541a53d40b602cc . "./code.cg2js/cg-class-slot-grid.fasl")
(#x499590dcf0bb390c25ca55fd2fa710ec . "./code.cg2js/cg-class-support.fasl")
(#x09b31bf5bd01fcc2c8497c7a0a63af51 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xa85bc495eb53bc3845c2c35d5f67dfe7 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x44d6b81a21124dd8aeba3c9a0431d214 . "./code.cg2js/cg-clipboard.fasl")
(#x82a6f65f296427893bd4fceeb2552d68 . "./code.cg2js/cg-color-dialog.fasl")
(#xee8be617c3387ec7b9fefbdfe5ac97e0 . "./code.cg2js/cg-combo-box.fasl")
(#xe1a0c747280d8cc0060c0fe5077adbca . "./code.cg2js/cg-common-control.fasl")
(#x0737110e8cbe4b1c898facddc16b0e81 . "./code.cg2js/cg-comtab.fasl")
(#x7c26d210c876ed74a2059543f6b46638 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x10ff0468c8171f81baea11c360404998 . "./code.cg2js/cg-curve.fasl")
(#x0a34660b0fbd16c1c7bc48e95f96b0b7 . "./code.cg2js/cg-dde-utils.fasl")
(#x5288c03abda0dbf50e52059ac8ad377b . "./code.cg2js/cg-dev.fasl")
(#x65ce96ba2716d9e5919c017ef4c6567a . "./code.cg2js/cg-dialog-item.fasl")
(#xb6adc8707b43bbf17a838475dcb42946 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xf42e63992c635cc06ac58420f1559b3f . "./code.cg2js/cg-directory-dialog.fasl")
(#x0f79353f27150e3806aed8cc8b062261 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x35b12af1d0d2fbc7191b11dc6b153827 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xbbf3e964402a3d34484a36b23ba7ee75 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xd7d01b98e0e02602b0aa2e095eb33c91 . "./code.cg2js/cg-drawable.fasl")
(#x27f67f5e86a6fe87f8c55c2e61413858 . "./code.cg2js/cg-dropping-outline.fasl")
(#x1d24c6dba41ef23ff42a079ba1c436bf . "./code.cg2js/cg-edit-in-place.fasl")
(#xeb8ec8ae8f6c7b9f63202a8108034430 . "./code.cg2js/cg-editable-text.fasl")
(#xd413767732032ae1880750034e1d690e . "./code.cg2js/cg-file-dialog.fasl")
(#xa34b84b6bb8a900949e728c347931f51 . "./code.cg2js/cg-fill-texture.fasl")
(#x62cf2cd06dff8bfbf97f29130a2c09d4 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xab6cf3d7204711a0f8496e3d6b73304f . "./code.cg2js/cg-font-dialog.fasl")
(#x721027aedb79c9a0ef1ce65a4bf09fb9 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x1a35924911456395578dd0dd242827b8 . "./code.cg2js/cg-get-pixmap.fasl")
(#x47139693b72ca5f315a5c28c1e6a5143 . "./code.cg2js/cg-get-position.fasl")
(#x8950ec60a84dafc51031e9101cf75263 . "./code.cg2js/cg-graphics-context.fasl")
(#x119f48e8dd9d182f3ec58afb228f9f42 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xba62411dfa85fe3470852ed0eddacdd8 . "./code.cg2js/cg-grid-widget.fasl")
(#xe40b7aac6e9c9b3db372e5835c96e8cb . "./code.cg2js/cg-group-box.fasl")
(#x56f0f7ce53e97e7d8b7bc550e63a1322 . "./code.cg2js/cg-header-control.fasl")
(#x625ea089f2f3ee496ce447bab3a27faf . "./code.cg2js/cg-hotspot.fasl")
(#xea03aaf05cd13bbad415ab546c95c418 . "./code.cg2js/cg-html-dialog.fasl")
(#xf9cc03dc3ac1bd4ccc13ca76d9c7e097 . "./code.cg2js/cg-html-widget.fasl")
(#xe32e98679885fc3d0bb8e7a85a18c472 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xd5e22ca5928d2db9e625c88463a7b309 . "./code.cg2js/cg-icon.fasl")
(#xb1d6e6fb3d7729b8f5d36be8a66fa3e0 . "./code.cg2js/cg-ie.fasl")
(#x0316a59f039f7f13aaaa28ad90eaa496 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x9847bf59c76ff2b3dc4c8d5e790c8a78 . "./code.cg2js/cg-intersect.fasl")
(#xeea6831f218bad426c937a5b95bae5c7 . "./code.cg2js/cg-item-list.fasl")
(#x6742cc0e97f0e14f6785503dbad7682e . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x1a754b18dc195bfc6cb96deaa1afc23c . "./code.cg2js/cg-lamp.fasl")
(#x4da339ba79d0f560197cd88e98637e9c . "./code.cg2js/cg-layout.fasl")
(#xf67756a318fe19422053c7f3b3367f8f . "./code.cg2js/cg-lettered-menu.fasl")
(#x06226002cdf49050049d694415aada8c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xddab8b8da96d33d41ab48c65a9ff3c6a . "./code.cg2js/cg-lisp-text.fasl")
(#xf509cb69ce9af24191f0d98d9261dd24 . "./code.cg2js/cg-lisp-widget.fasl")
(#x830544e448fcbccb901e35f9f018ded6 . "./code.cg2js/cg-list-view.fasl")
(#xbf37c8da74e8a2277ceefbd97d870a26 . "./code.cg2js/cg-mci.fasl")
(#x9a324288fd774dfe818650daf5f54727 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xc160c6cfb5175ff0f10480a26ffb32f6 . "./code.cg2js/cg-menu.fasl")
(#xa541f776d16f6639bf53b3e2075930c4 . "./code.cg2js/cg-message-dialog.fasl")
(#x06bf0396c71b11385b34236f62db8a85 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x88a16d2c25a1fcffba31ea3903dd0e8c . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xb29a0e4ed2dc1080b70d1d0c96e4cc93 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x7de118f56d00ec905404eb2ccbb2f2c8 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x372105bcc07928af8ad4ee6c0781b1a2 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x341ead81f492eb5e02bb0ee5c22bff8b . "./code.cg2js/cg-nodes.fasl")
(#x7334315a6f13de557c412c6466e8cbdf . "./code.cg2js/cg-object-editor-layout.fasl")
(#x7f0f989c86751589a8438769498284d7 . "./code.cg2js/cg-object-editor.fasl")
(#x567e5f6a465033a14656490793e999bf . "./code.cg2js/cg-ocx.fasl")
(#xddcfd5d35db9dc0ad8eedc2267fd81db . "./code.cg2js/cg-os-widget.fasl")
(#x8ec180a220e00dbb3f743c22c989ef05 . "./code.cg2js/cg-os-window.fasl")
(#x1eaaca32e3edf34a5c48b5b1b4bbd700 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x0aefc425c51bd744abff787cae44dccd . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x0f5074b5bfc1f9f3237fe748d7c08d5a . "./code.cg2js/cg-outline.fasl")
(#x36ca68d2fcca63fbcb7d2cadbd30f3c3 . "./code.cg2js/cg-palette.fasl")
(#x55f9c8c8f272dd6a703be1193a258498 . "./code.cg2js/cg-paren-matching.fasl")
(#xeb0859dae3494f903d7371dac16ea5ad . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x453fc0a53c905d97b06e46e1e43d9abc . "./code.cg2js/cg-picture-widget.fasl")
(#x2324e51bb6cd49941bd44378a83b3bf8 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x1238acc85b6f85ba32cebe7ea2c139ff . "./code.cg2js/cg-pixmap-printing.fasl")
(#xdecdab4ee45e698e27e18788b59349dd . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xef84c1588aaad69d0973ff69b080b697 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x5dc49214b4b52777c4ef0b6bdbe0615f . "./code.cg2js/cg-pixmap.fasl")
(#x1ba947b30b863a1b03b905238ac0ed1d . "./code.cg2js/cg-plot-widget.fasl")
(#x0296e18a36443fb826cd3c2bb73b8890 . "./code.cg2js/cg-printing.fasl")
(#x3c16b0c55fa31e73bbd51b9d4421e9a8 . "./code.cg2js/cg-progress-indicator.fasl")
(#x8a7430f7120e83e1d08d47faf82fb264 . "./code.cg2js/cg-project-window.fasl")
(#x5b617c9127b341a6b98577ab94f13169 . "./code.cg2js/cg-property.fasl")
(#x9908bbf35974116d88ba26f6ae7270e5 . "./code.cg2js/cg-radio-button.fasl")
(#x79a027b470a0edcd39de87cf7144a7ea . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x5fdd9f7eb75adba9675c07c7f04466f5 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xaf4f6556fd5d8129b64d1fff79437cc1 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xc5ad044d74ee82056c8cbb8a7b10e65a . "./code.cg2js/cg-rich-edit.fasl")
(#x968fb7f8e5071d7d69dac4785064fe0e . "./code.cg2js/cg-sample-file-menu.fasl")
(#xfd26504964121a7b3b0bc0929840d536 . "./code.cg2js/cg-scaling-stream.fasl")
(#xb66cca6e7a9c19b39d8b50e1d32795f7 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x4d497e35616407ed31634eab5101bd75 . "./code.cg2js/cg-scroll-bar.fasl")
(#x9a0ef4bca834eafb58b3ea41fb018da5 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xd516505ae0617801eee53bd9f46a6614 . "./code.cg2js/cg-selected-object.fasl")
(#xe628bb5d2b0644324f9c62bff0ab5c47 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x43d3b4f679625d5c74134a42319348fb . "./code.cg2js/cg-split-bar.fasl")
(#x299d8a32921d58d3652fbb39ab29f5e5 . "./code.cg2js/cg-static-text.fasl")
(#x86a24a724fbcee7f657af0ef17f80bd1 . "./code.cg2js/cg-status-bar.fasl")
(#x508b988c936e9b782e61964fcb83172f . "./code.cg2js/cg-string-dialog.fasl")
(#x7902404f9a71cc8ce708ffa15bf2e748 . "./code.cg2js/cg-tab-control.fasl")
(#xc1107114b1b2fbc6be3cf4da29e08ce8 . "./code.cg2js/cg-template-string.fasl")
(#x9ece54f37cddd5ebe78b075515ccc2fd . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x64c2b1dcaa6ce00e412b9937f2e954e5 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xce44a9c674a47ac71d1ea84cfcdebaef . "./code.cg2js/cg-text-edit-pane.fasl")
(#x485beaf353f5d40fad4256d009b3f06d . "./code.cg2js/cg-text-or-combo.fasl")
(#x55f37970205499772f653a2df79ecdb0 . "./code.cg2js/cg-text-widget.fasl")
(#xaa4c1efe829aac60fd4090a1a020e9de . "./code.cg2js/cg-timer.fasl")
(#xac304a7b7b69f5af52bf92e7b53a3d81 . "./code.cg2js/cg-toggling-widget.fasl")
(#x4dd1afb3d296263e2f109220961abe22 . "./code.cg2js/cg-toolbar.fasl")
(#x3e2591d57e5413b43bfe5931bde89a55 . "./code.cg2js/cg-tooltip.fasl")
(#x31a7822bf7c25d594690e4969c299e33 . "./code.cg2js/cg-touch.fasl")
(#x5abb0b6ff0ce17e0f3b8cf58b3c4effb . "./code.cg2js/cg-trackbar.fasl")
(#x628cb8ba3b22eaa875d54b0d2a93c7d5 . "./code.cg2js/cg-tray.fasl")
(#xa0a06f61069d1dbcde56d5dc5597cba2 . "./code.cg2js/cg-up-down-control.fasl")
(#x25167e8609a1ced63282a1f071906ace . "./code.cg2js/cg-utility-dialog.fasl")
(#x0c1eecf1deb79206983f420002892a07 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x2458b7b25a5932deff76e7742f31292a . "./code.cg2js/cg-web-browser.fasl")
(#xcd56f3bddbfd3a32406a34c1c618a0c6 . "./code.cg2js/cg-wrap-string.fasl")
(#x5fa73f9d28fd26ec8d5e89c2c123c074 . "./code.cg2js/cg-yes-no-list.fasl")
(#x8c3aa12e8b11722cecd772c20a45ca84 . "./code.cg2js/cg-yes-no-string.fasl")
(#xb2811f9275a25b31866ba0f930565b49 . "./code.cg2js/cg.fasl")
(#xf396ca2bd035f1c366f0e239a35b1884 . "./code.idejs/ide-acache.fasl")
(#x58a5d74d35f9423676ad9198bb81b08f . "./code.idejs/ide-apropos.fasl")
(#xaae1df7d0792abc39ec083230ec573fe . "./code.idejs/ide-base.fasl")
(#x746da9e8da6d8912a7de6765e5845693 . "./code.idejs/ide-class-browser.fasl")
(#x421a9e5c8bac1f063d9eb02021310153 . "./code.idejs/ide-class-grapher.fasl")
(#xed97c5c380b2e82399f81c6382d981a4 . "./code.idejs/ide-clipboard.fasl")
(#x3becdc15b72717b8901f794803257be2 . "./code.idejs/ide-dev.fasl")
(#xd87076be79cbe8956cdb2814ce1dcd95 . "./code.idejs/ide-editor.fasl")
(#x80a329e7adc63e619c81126df05608f5 . "./code.idejs/ide-find-definitions.fasl")
(#xfed9222da1684d79873d8cb466843d0b . "./code.idejs/ide-find-in-files.fasl")
(#x6a911871983e61252ba8b6459336dddf . "./code.idejs/ide-form.fasl")
(#xf7dc1f26669fbf958f11aad013892b61 . "./code.idejs/ide-grapher.fasl")
(#x19aeea2dd9fcc6cd065917ee9548f356 . "./code.idejs/ide-inspector.fasl")
(#xc2c66d4f7b0efc062090b8cf7f9369d4 . "./code.idejs/ide-intro.fasl")
(#x074508b31749f1db6213062c0a7b69de . "./code.idejs/ide-keyshort.fasl")
(#x38e80cfe0273bd31655cdee6ae5c6639 . "./code.idejs/ide-navigator.fasl")
(#x76ce0b63d942118606216b80f0477d91 . "./code.idejs/ide-package-list.fasl")
(#x5dd68a125f2ca4e360bae49e160c6e3b . "./code.idejs/ide-patch-dialog.fasl")
(#x35fadd2f06f3c291c76a3bdb23e5c631 . "./code.idejs/ide-process-browser.fasl")
(#xf6474ae32877a03c895245af245590ab . "./code.idejs/ide-project.fasl")
(#x0574e5bca3530ddb772fbd92996f1bc5 . "./code.idejs/ide-trace.fasl")
(#x56e4a347a4e45ac95d3b316dd402e5ed . "./code.idejs/ide-window-list.fasl")
(#x720e74c183d46fcdfefcb7264eb518e8 . "./code.idejs/ide.fasl")
(#xf456f3c7874f303a35ad67e1f3673be9 . "./cgjs.html")
(#x85aa0e1902236a67f67453f2c909ea34 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
